using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// BokehCircleEditor v1.0
//
// Allows you to edit some parameters of Bokeh particles (circle type)

namespace BokehCircleEditorPlugin
{
    public class BokehCircleEditor : MVRScript
    {
		public JSONStorableColor BokehMainColor;
		public JSONStorableFloat BokehMainOpacity;
		public JSONStorableFloat BokehType;
		public JSONStorableFloat BokehMainMaxParticles;
		public JSONStorableFloat BokehRadius;
		public JSONStorableFloat BokehMainSizeMin;
		public JSONStorableFloat BokehMainSizeMax;
		
		public JSONStorableBool BokehMainUseRandomSeed;
		public JSONStorableFloat BokehMainManualSeed;
		
		public JSONStorableFloat ParticlesSortingPriority;
		
		public Transform ParticleRoot;
		private ParticleSystem prtComponent;
		private ParticleSystemRenderer prtRendererComponent;
		private string objectNameSearchString = "_editable";

		private bool seInitialized = false;
		
		// Security
		private int initLoops = 0;
		private int maxInitLoops = 3600;
		
		public override void Init()
        {			 
            try
            {
				if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Please add BokehCircleEditor on an Custom Unity Asset with a compatible particle system.");
                    return;
                }

				HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				BokehMainColor = new JSONStorableColor("Bokeh color", hsvc, MainColorCallback);
				RegisterColor(BokehMainColor);
				CreateColorPicker(BokehMainColor);
				
				BokehMainOpacity = new JSONStorableFloat("Bokeh opacity", 0.5f, MainOpacityCallback, 0f, 1f);
				RegisterFloat(BokehMainOpacity);
				CreateSlider(BokehMainOpacity, false);
				
				BokehType = new JSONStorableFloat("Bokeh type", 0f, (val) => { BokehType.valNoCallback = Mathf.Round(val); TypeCallback( BokehType.val ); }, 0f, 31f);
				RegisterFloat(BokehType);
				CreateSlider(BokehType, false);
				
				BokehMainMaxParticles = new JSONStorableFloat("Max Bokeh", 50, MainMaxParticlesCallback, 1, 1000);
				RegisterFloat(BokehMainMaxParticles);
				CreateSlider(BokehMainMaxParticles, false);

				BokehRadius = new JSONStorableFloat("Bokeh radius", 1.5f, ( float val ) => { RadiusCallback( val ); }, 1f, 10f);
				RegisterFloat(BokehRadius);
				CreateSlider(BokehRadius, false);
				
				BokehMainSizeMin = new JSONStorableFloat("Min Size", 10, MainSizeMinCallback, 1, 100);
				RegisterFloat(BokehMainSizeMin);
				CreateSlider(BokehMainSizeMin, false);
				
				BokehMainSizeMax = new JSONStorableFloat("Max Size", 12, MainSizeMaxCallback, 1, 100);
				RegisterFloat(BokehMainSizeMax);
				CreateSlider(BokehMainSizeMax, false);
					
				CreateSpacer();
				
				BokehMainUseRandomSeed = new JSONStorableBool("Use random seed", true, MainUseRandomSeedCallback);
				RegisterBool(BokehMainUseRandomSeed);
				CreateToggle(BokehMainUseRandomSeed, false);
				
				BokehMainManualSeed = new JSONStorableFloat("Custom seed", 0, (val) => { BokehMainManualSeed.valNoCallback = Mathf.Round(val); MainManualSeedCallback( BokehMainManualSeed.val ); }, 0.0f, 1000000.0f);
				RegisterFloat(BokehMainManualSeed);
				CreateSlider(BokehMainManualSeed, false);
				
				CreateSpacer();
								
				ParticlesSortingPriority = new JSONStorableFloat("Particles priority", 0, SortingPriorityCallback, -1f, 1f);
				RegisterFloat(ParticlesSortingPriority);
				CreateSlider(ParticlesSortingPriority, false);
							
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>BokehCircleEditor</b> is a tiny plugin that allows you to edit some parameters of the Bokeh particles inside this package\n\n" +
					"Just test the parameters on the left, and the particle will update in realtime.\n\n" +
					"With multiple particle systems, if you want some particles to be in front of the others use <b>Particles priority.</b> Lower values are displayed in the front, higher values are displayed in the back.\n\n" +
					"<b>For creators and screenshot/still artists :</b> Be careful. I cannot explain why, but the vanilla screenshot feature of VAM does not render exactly the same as what you see on the realtime viewport. " +
					"In my case, I used virtual resolution to upscale the game and make higher res screenshots.\n\n" +
					"<b>IMPORTANT :</b> If you switch particles in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 1100.0f;
				

			}
            catch(Exception e)
            {
                SuperController.LogError("BokehCircleEditor - Exception caught: " + e);
            }
		}
		
		void Start()
        {

        }
		
		void Update()
        {
			// If we haven't made over 1min of init tests, we aren't initialized and the particle system (CUA) is still compatible, try to initialize
			if( initLoops < maxInitLoops && seInitialized == false ) {
				initLoops++;
				InitDE();
			}
			
			if( initLoops == maxInitLoops ) {
				logError("BokehCircleEditor : This CUA particle could not be initialized properly after 3600 tries. Please select a compatible particle and reload the script.");
				initLoops++; // To avoid spamming the log
			}
		}
		
		private bool InitDE() {
			if( seInitialized == true ) return true;
			
			ParticleRoot = getObjectRoot( containingAtom.reParentObject );
			if( ParticleRoot == null ) {
				return false;
			}
			
			prtComponent = ParticleRoot.GetComponent<ParticleSystem>();
			if( prtComponent ) {
				prtRendererComponent = ParticleRoot.GetComponent<ParticleSystemRenderer>();
				seInitialized = true;
				
				MainColorCallback( BokehMainColor );
				MainOpacityCallback( BokehMainOpacity.val );
				TypeCallback( BokehType.val );
				
				MainMaxParticlesCallback( BokehMainMaxParticles.val );
				
				RadiusCallback( BokehRadius.val );
				MainSizeMinCallback( BokehMainSizeMin.val );

				MainManualSeedCallback( BokehMainManualSeed.val );
				
				SortingPriorityCallback( ParticlesSortingPriority.val );
				
				logDebug("BokehCircleEditor : Bokeh editor initialized properly");
							
				return true;
			} else {
				logError("BokehCircleEditor : your CUA has a transform with _editable but does not have a ParticleSystem component attached.");
				initLoops = maxInitLoops; // We don't have a particle system, we consider the initialization failed
				return false;
			}
		}
		
		private Transform getObjectRoot( Transform parent ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(objectNameSearchString) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}

		protected void MainColorCallback(JSONStorableColor color) {
			if( prtComponent ) {
				Color newColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, BokehMainOpacity.val);
				var prtMain = prtComponent.main;
				prtMain.startColor = newColor;
				prtComponent.Clear();
				prtComponent.Play();
			}
		}
		
		protected void MainOpacityCallback(float opacity) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startColor = new Color(BokehMainColor.colorPicker.currentColor.r, BokehMainColor.colorPicker.currentColor.g, BokehMainColor.colorPicker.currentColor.b, opacity);
				prtComponent.Clear();
				prtComponent.Play();
			}
		}
		
		protected void TypeCallback(float type) {
			if( prtComponent ) {
				var prtTSA = prtComponent.textureSheetAnimation;
				prtTSA.startFrame  = 0.03125f * type;
				prtComponent.Clear();
				prtComponent.Play();
			}
		}
		
		protected void MainMaxParticlesCallback(float maxprts) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.maxParticles = (int)maxprts;
				var prtEmission = prtComponent.emission;
				prtEmission.rate = maxprts;
				prtComponent.Clear();
				prtComponent.Play();
			}
		}
		
		protected void RadiusCallback(float radius ) {
			if( prtComponent ) {
				var prtShape = prtComponent.shape;
				prtShape.radius = radius;
				
				prtComponent.Clear();
				prtComponent.Play();
			}
		}
		
		protected void MainSizeMinCallback(float minsize) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startSize = new ParticleSystem.MinMaxCurve(minsize, BokehMainSizeMax.val);
				
				prtComponent.Clear();
				prtComponent.Play();
			}
		}
		
		protected void MainSizeMaxCallback(float maxsize) {
			if( prtComponent ) {
				var prtMain = prtComponent.main;
				prtMain.startSize = new ParticleSystem.MinMaxCurve(BokehMainSizeMin.val, maxsize);
				
				prtComponent.Clear();
				prtComponent.Play();
			}
		}

		protected void MainUseRandomSeedCallback(bool state) {
			if( prtComponent ) {
				// IMPORTANT, always stop and clear the particle system before changing the seed properties
				prtComponent.Stop();
				prtComponent.Clear();
				
				prtComponent.useAutoRandomSeed = state;
				
				// Only setting the seed when you don't want to use it because Unity disable the random if the value is set
				if( state == false ) {
					prtComponent.randomSeed = uint.Parse(BokehMainManualSeed.val.ToString());
				}
				
				prtComponent.Play();
			}
		}
		
		protected void MainManualSeedCallback(float randomSeed) {
			if( prtComponent ) {
				if( BokehMainUseRandomSeed.val == false ) {
					// IMPORTANT, always stop and clear the particle system before changing the seed properties
					prtComponent.Stop();
					prtComponent.Clear();
					
					prtComponent.useAutoRandomSeed = false;
					prtComponent.randomSeed = uint.Parse(randomSeed.ToString());
					
					prtComponent.Play();
				}
			}
		}
		
		protected void SortingPriorityCallback(float priority) {
			if( prtRendererComponent ) {
				prtRendererComponent.sortingFudge = priority;
			}
		}
		
		// **************************
		// Local Tools
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
	}
}